@@ -195,6 +195,24 @@ sub contains {
   $POINTER->contains($self->data, @_);
 }
 
+=head2 fresh
+
+  $self = $self->fresh;
+
+Calling this method will force the next relationship call to return fresh
+data from database instead of cached. Example:
+
+  $self->fresh->cats(sub {
+    my($self, $err, $cats) = @_;
+  });
+
+=cut
+
+sub fresh {
+  $_[0]->{fresh} = 1;
+  $_[0];
+}
+
 =head2 get
 
   $any = $self->get('/json/2/pointer');
@@ -450,6 +468,14 @@ L<Mojo::JSON>.
 
 sub TO_JSON { shift->data }
 
+sub _cache {
+  my $self = shift;
+  my $cache = $self->{cache} ||= {};
+
+  return $cache->{$_[0]} if @_ == 1;    # get
+  return $cache->{$_[0]} = $_[1];       # set
+}
+
 =head1 SEE ALSO
 
 L<Mojolicious>, L<Mango>, L<Mandel>
