@@ -114,25 +114,35 @@ sub monkey_patch {
 }
 
 sub _monkey_patch_all_method {
-  my $self   = shift;
-  my $search = $self->search_method_name;
+  my $self     = shift;
+  my $search   = $self->search_method_name;
+  my $accessor = $self->accessor;
 
   Mojo::Util::monkey_patch(
     $self->document_class,
-    $self->accessor,
+    $accessor,
     sub {
       my ($doc, $cb) = @_;
+      my $cached = delete $doc->{fresh} ? undef : $doc->_cache($accessor);
 
       # Blocking
-      return $doc->$search->all unless $cb;
+      unless ($cb) {
+        return $cached if $cached;
+        return $doc->_cache($accessor => $doc->$search->all);
+      }
 
-      # Non-blocking
-      $doc->$search->all(
-        sub {
-          my ($collection, $err, $objs) = @_;
-          $doc->$cb($err, $objs);
-        }
-      );
+      if ($cached) {
+        $doc->$cb('', $cached);
+      }
+      else {
+        $doc->$search->all(
+          sub {
+            my ($collection, $err, $objs) = @_;
+            $doc->_cache($accessor => $objs) unless $err;
+            $doc->$cb($err, $objs);
+          }
+        );
+      }
 
       return $doc;
     }
@@ -144,12 +154,14 @@ sub _monkey_patch_all_method {
 sub _monkey_patch_add_method {
   my $self          = shift;
   my $foreign_field = $self->foreign_field;
+  my $accessor      = $self->accessor;
 
   Mojo::Util::monkey_patch(
     $self->document_class,
     $self->add_method_name,
     sub {
       my ($doc, $obj, $cb) = @_;
+      my $cached = $doc->_cache($accessor);
 
       if (ref $obj eq 'HASH') {
         $obj = $self->_related_model->new_collection($doc->connection)->create($obj);
@@ -159,6 +171,7 @@ sub _monkey_patch_add_method {
 
       # Blocking
       unless ($cb) {
+        push @$cached, $obj if $cached;
         $obj->save;
         $doc->save;
         return $obj;
@@ -174,6 +187,7 @@ sub _monkey_patch_add_method {
         sub {
           my ($delay, $o_err, $d_err) = @_;
           my $err = $o_err || $d_err;
+          push @$cached, $obj if !$err and $cached;
           $doc->$cb($err, $obj);
         },
       );
